@@ -9,10 +9,19 @@ require Exporter;
 
 our @ISA = qw(Exporter);
 
-our $VERSION = '0.02';
+our $VERSION = '0.03';
+
+our $urls = {
+	ZA => 'http://bulksms.2way.co.za:5567',
+	UK => 'http://www.bulksms.co.uk:5567',
+	ES => 'http://bulksms.com.es:5567',
+	DE => 'http://bulksms.de:5567',
+	US => 'http://usa.bulksms.com:5567',
+	INT => 'http://bulksms.vsms.net:5567'
+};
 
 our $default_options = {
-	base_url => 'http://bulksms.2way.co.za:5567/eapi/submission/send_sms/2/2.0',
+	country => 'ZA',
 };
 
 our @bulksms_send_options = qw(sender msg_class dca want_report routing_group source_id repliable strip_dup_recipients stop_dup_id send_time send_time_unixtime scheduling_description test_always_succeed test_always_fail allow_concat_text_sms oncat_text_sms_max_parts);
@@ -62,6 +71,9 @@ sub new {
 	# Is there a proxy ?
 	$ENV{http_proxy} = $ref->{http_proxy} if ($ref->{http_proxy} ne '');
 	
+	# Wich base URL to use ?
+	$ref->{base_url} = $urls->{$ref->{country}};
+	
 	return 0 if $error;
 	
 	bless ($ref, $class);
@@ -72,12 +84,10 @@ sub send_sms {
 	my $message = shift @_;
 	my @recipients = @_;
 	
-	if (!$message) {
-		# I'm still undecided as what to do here ? 
-		# To not allow blank SMS messages, uncomment 2 lines below
-		#$this->{error} = "Message is empty!\n";
-		#return 0;
-	}
+	#if (!$message) {
+	#	$this->{error} = "Message is empty!\n";
+	#	return 0;
+	#}
 
 	foreach my $number (@recipients) {
 		next if $number =~ /[a-zA-Z]/;
@@ -85,21 +95,18 @@ sub send_sms {
 		$number =~ s/\D//g;
 		
 		if ($this->{sa_numbers_only} > 0 && $number =~ /^(27|0[78])/) {
-			$number =~ s/^0(82|82|84|72|73|76|79)(\d+)/27$1$2/;
+			$number =~ s/^0(82|83|84|72|73|76|79|78)(\d+)/27$1$2/;
 		} else {
 			next;
 		}
 		
-	}
-	
-	if ($recipients[0] eq '') {
-		# TODO: Error checking on all numbers in list
-		$this->{error} = "Recipient is empty!\n";
-		return 0;
-	}
-	
-
+		if ($number eq '') {
+			$this->{error} = "One of the recipient numbers is invalid: $number";
+			return 0;
+		}
 		
+	}
+	
 	my $args;
 	
 	# Extract all the BulkSMS options
@@ -112,25 +119,21 @@ sub send_sms {
 	$args->{msisdn} = join(',', @recipients);
 	$args->{message} = $message;
 	
-	my @tmp = $this->http_post($args) || ($this->send_to_log("WARN: Could not do http_post() for send_sms(): " . $this->{error}) && return 0);
+	my $url = $this->{base_url} . '/eapi/submission/send_sms/2/2.0';
+	my @tmp = $this->http_post($url, $args) || ($this->send_to_log("WARN: Could not do http_post() for send_sms(): " . $this->{error}) && return 0);
 	
 	$this->send_to_log("Message sent to " . join(',', @recipients)) if $this->{verbose} > 0;
 	
-	return pop(@tmp);
+	#return pop(@tmp);
+	return join("*", @tmp);
 }
 
 sub  get_credits {
 	my $this = shift @_;
 	
-	my $old_base_url = $this->{base_url};
-	$this->{base_url} = 'http://bulksms.2way.co.za:5567/eapi/user/get_credits/1/1.1';
-	
-	my @tmp = $this->http_post();
-	
+	my $url = $this->{base_url} . '/eapi/user/get_credits/1/1.1';
+	my @tmp = $this->http_post($url);
 	my ($status, $balance) = split /\|/, pop(@tmp) || ($this->send_to_log("WARN: Could not do http_post() for get_credits(): " . $this->{error}) && return -1);
-	
-	$this->{base_url} = $old_base_url;
-	
 	return $balance;
 }
 
@@ -138,48 +141,56 @@ sub get_inbox {
 	my $this = shift @_;
 	my $last_retrieved_id = shift @_;
 	
-	my $old_base_url = $this->{base_url};
-	
-	$this->{base_url} = 'http://bulksms.2way.co.za:5567/eapi/reception/get_inbox/1/1.0';
-	
+	my $url = $this->{base_url} . '/eapi/reception/get_inbox/1/1.0';
 	my $args = {last_retrieved_id => $last_retrieved_id};
-	
-	my @tmp = $this->http_post($args) || ($this->send_to_log("WARN: Could not do http_post() for get_inbox(): " . $this->{error}) && return 0);
-	
-	$this->{base_url} = $old_base_url;
-	
+	my @tmp = $this->http_post($url, $args) || ($this->send_to_log("WARN: Could not do http_post() for get_inbox(): " . $this->{error}) && return 0);
 	return @tmp;
 }
 
 sub get_report {
 	my $this = shift @_;
 	my $batch_id = shift @_;
-	
+  	my @report;
+  	my $marker = 0;
+  	
 	if (!$batch_id) {
 		$this->{error} = "batch_id was not specified\n";
 		return 0;
 	}
 	
-	my $old_base_url = $this->{base_url};
-	
-	$this->{base_url} = 'http://bulksms.2way.co.za:5567/eapi/status_reports/get_report/2/2.0?';
+	my $url = $this->{base_url} .= '/eapi/status_reports/get_report/2/2.0?';
 	
   	my $args = { batch_id => $batch_id, optional_fields => 'body,completed_time,created_time,credits,origin_address,source_id' };
-  	my @tmp = $this->http_post($args) || ($this->send_to_log("WARN: Could not do http_post() for get_report(): " . $this->{error}) && return 0);
   	
-  	$this->{base_url} = $old_base_url;
+  	my @tmp = $this->http_post($url, $args) || ($this->send_to_log("WARN: Could not do http_post() for get_report(): " . $this->{error}) && return 0);
   	
-	return @tmp;
+  	foreach my $row (@tmp) {
+  		if (/(\d+)\|([\w\s\d]+)$/) {
+  			if ($1 != 0) {
+  				$this->{error} = "$1: $2";
+  				return 0;
+  			} else {
+  				$marker = 1;
+  			}
+  		}
+  		
+  		if ($marker) {
+  			push (@report, $row);
+  		}
+  	}
+  	
+	return @report;
 }
 
 sub http_post {
 	my $this = shift @_;
+	my $url = shift @_;
 	my $args = shift @_;
 	
 	my $timeout = $this->{timeout} || 30;
 	
 	my $uagent = LWP::UserAgent->new(timeout => $timeout);
-  	my $request = HTTP::Request->new(POST => $this->{base_url});
+  	my $request = HTTP::Request->new(POST => $url);
   	$request->content_type('application/x-www-form-urlencoded');
   	
   	my $content = 'username=' . $this->{username} . '&password=' . $this->{password};
@@ -239,7 +250,7 @@ sub _parse_config {
 		s/#.*//;		# Strip away all comments
 		
 		s/^\s+//;		# Remove leading...
-		s/\s+$//;		# and trailing whitespace
+		s/\s+$//;		# ...and trailing whitespace
 		
 		s/\s*=\s*/=/;		
 		
@@ -268,9 +279,9 @@ Net::SMS::2Way - BulkSMS API
   
   my $sms = Net::SMS::2Way->new('config' => '/etc/SMS_Options.cfg');
   
-  $sms->send_sms('Hello World!', '0821234567');
+  $sms->send_sms('Hello World!', '27821234567');
   
-  $sms->send_sms('Hello World!', ['0821234567','0831234567','0841234567']);
+  $sms->send_sms('Hello World!', ['27821234567','27831234567','27841234567']);
   
   $sms->send_sms('Hello World!', @recipients);
   
@@ -278,7 +289,8 @@ Net::SMS::2Way - BulkSMS API
 =head1 DESCRIPTION
  
  This module allows you to send SMS text messages using the HTTP API that is available from BulkSMS 
- in South Africa (http://bulksms.2way.co.za)
+ in South Africa (http://bulksms.2way.co.za) but can be configured to work with all the BulkSMS sites. 
+ You can find a list of them at http://www.bulksms.com/selectwebsite.htm
 
 =head2 The BulkSMS API
 
@@ -296,23 +308,15 @@ Net::SMS::2Way - BulkSMS API
  send_batch
  quote_sms
  public_add_member 
- public_remove_member  
- 
- I will incorporate the FTP API as soon as the HTTP implementation is considered to be stable.
+ public_remove_member
 
 =head1 REQUIREMENTS
 
-=item *
+ 1.) You need to register at one of the BulkSMS sites (http://www.bulksms.com/selectwebsite.htm) and have some credits available.
 
- You need to register at http://bulksms.2way.co.za and have some credits available.
-
-=item *
-
- You will need the LWP modules installed. This module was tested with version 5.75
+ 2.) You will need the LWP modules installed. This module was tested with version 5.75
  
-=item *
-
- An internet connection.
+ 3.) An internet connection.
 
 =head1 METHODS
 
@@ -327,11 +331,20 @@ Net::SMS::2Way - BulkSMS API
  config_file: The path to config file
  verbose: Write debug information to logfile
  logfile: The path to the logfile. Default is $0.log (To turn off logging (override verbose) set this option to -1)
- username: The username you registered with at http://bulksms.2way.co.za
- password: The password you registered with at http://bulksms.2way.co.za
+ username: The username you registered with at http://bulksms.2way.co.za (or one of the other sites).
+ password: The password you registered.
  http_proxy: Which web-proxy to use e.g. http://10.0.0.1:8080
- sa_numbers_only: Set this to 1 if you only want to send to South African mobile numbers
- 
+ sa_numbers_only: Set this to 1 if you only want to send to South African mobile numbers.
+ country: Which base url to use, derived from which regional BulkSMS site you want to use. 
+ Can be one of ZA, UK, ES, DE, US, INT. default is ZA.
+
+	ZA => South Africa (API Base URL: http://bulksms.2way.co.za:5567)
+	UK => United Kingdom (API Base URL: http://www.bulksms.co.uk:5567)
+	ES => Spain (API Base URL: http://bulksms.com.es:5567)
+	DE => Germany (API Base URL: http://bulksms.de:5567)
+	US => United States (API Base URL: http://usa.bulksms.com:5567)
+	INT => International (API Base URL http://bulksms.vsms.net:5567)
+
  See the bulkSMS API for the meaning of the options below (http://bulksms.2way.co.za/docs/eapi/submission/send_sms/):
 
  sender 
@@ -365,14 +378,13 @@ Net::SMS::2Way - BulkSMS API
 
   # My config
   verbose = 1
+  country = UK
   logfile = /usr/local/sms/sms.log
   sender = 27841234567
   username = johnny
   password = S3kR3t
   want_report = 1
 
-=item *
- 
  By default a log file will be created and failures or serious errors will be logged, no matter what the verbose option is set to. 
  If you do not want any logs at all, you must set logfile to -1
 
@@ -419,17 +431,37 @@ Net::SMS::2Way - BulkSMS API
   
   On failure, a return value of -1 can be expected with the reason in $sms->{error}
   
-=head2 get_reports()
+=head2 get_report()
+
+ get_report(INTEGER) - Takes 1 argument which is batch_id i.e. the id returned by a successful submission to send_sms(). It's fully explained at http://bulksms.2way.co.za/docs/eapi/status_reports/get_report/.
+ 
+ Return Values: 0 on failure with the error message in $sms->{error}
+ 
+ Returns an array on success. Each element has a string which has the format of:
+ 
+ msisdn|status_code|body|completed_time|created_time|credits|origin_address|source_id
+
+=head2 get_report_hash_ref()
 
  TODO: Finish documentation ASAP!!!
- 
+
 =head2 get_inbox()
 
  TODO: Finish documentation ASAP!!!
  
+=head1 SUPPORT
+
+ Bugs: You can email the author directly at lee@kode.co.za
+
+ Commercial support is available.
+ 
+=head1 THANKS
+
+ Many thanks to the guys at BulkSMS for their helpful feedback and comments.
+ 
 =head1 AUTHOR
 
- Lee Engel, lee@kode.co.za, http://www.easitext.net
+ Lee Engel, lee@kode.co.za
 
 =head1 COPYRIGHT AND LICENSE
 
